#include "taskFlyport.h"
#include "1wire.h"

#define xDelay(x) (portTickType)(x/portTICK_RATE_MS)


// Defines the server to be accessed
static ROM char ServerName[] =   "api.pachube.com";
// Defines the port to be accessed
static ROM char ServerPort[] = "80";
// Defines the URL to be requested, It containes the feed number to use.
static ROM char RemoteURL[] = "/v2/feeds/<<<put your own feed number here>>>.csv";
// Defines the Pachube API Key to use
static ROM char PachubeKey[] = "<<<put your own key here>>>;
// Defines the URL to be requested, It containes the feed number to use.
static ROM char GoodHTTPResponse[] = "200 OK";


double temp=0.0;
int WFStatus_save;


void FlyportTask()
{
	int i=0;
	static TCP_SOCKET   MySocket = INVALID_SOCKET;
	char buf[50];
	char tmpString[200 + strlen(ServerName) + strlen(RemoteURL) + strlen(PachubeKey)];
	BOOL err = FALSE;
	
	// this is a 1st dummy read because 1st read contains sometimes garbage data
	vTaskSuspendAll();			// Entering critical section
	temp=Read_Temperature();
	xTaskResumeAll();			// Exiting critical section

	WFConnect(WF_DEFAULT);			// start connection
	while (WFStatus != CONNECTED);	// wait until connected to WiFi

	
	#if defined(STACK_USE_UART)
	UARTWrite(1,"Flyport say: hello world!\r\n");
	#endif

	
	while(1)
	{         
		vTaskDelay( 1000 );			// wait 1000x10ms=10s

		vTaskSuspendAll();			// Entering critical section
		temp=Read_Temperature();
		xTaskResumeAll();			// Exiting critical section
		sprintf(buf,"DS1820 = %.2f C\r\n",temp);
		#if defined(STACK_USE_UART)
		UARTWrite(1,buf);
		#endif
	  
		sprintf(buf,"0,%.2f\r\n",temp);		// prepare pachube data
		IOPut(o4,ON);						// turn on LED to signal start of connection to pachube
		#if defined(STACK_USE_UART)
		UARTWrite(1,"Connecting to Pachuba...\r\n");
		#endif

		// Connect a socket to the remote TCP server
		MySocket = TCPClientOpen(ServerName, ServerPort);
      
		// Abort operation if no TCP socket is available
		if(MySocket == INVALID_SOCKET) 
		{
			#if defined(STACK_USE_UART)
			UARTWrite(1,"INVALID_SOCKET\r\n");
			#endif
			continue;
		}

		#if defined(STACK_USE_UART)
		UARTWrite(1,"Checking connection.\r\n");
		#endif

		// Make sure it connects
		i = 10;
		while (i>0 && !TCPisConn(MySocket))
		{
			vTaskDelay( xDelay(500) );
			i--;
		}
		if(i==0) 
		{
			#if defined(STACK_USE_UART)
			UARTWrite(1,"Connection failed.\r\n");
			#endif
			// Close the socket so it can be used by other modules
			TCPClientClose(MySocket);
			MySocket = INVALID_SOCKET;
			// We are in an error condition, so light up the Flyport led
			err = TRUE;
			IOPut(o4,ON);
			continue;
		}

		// Place the Pachube request into the transmit buffer.
		sprintf( tmpString,
        "PUT %s HTTP/1.1\r\nHost: %s\r\nAccept: */*\r\nContent-Type: text/csv\r\nX-PachubeApiKey:%s\r\nContent-Length: %d\r\nConnection: close\r\n\r\n%s",
         RemoteURL, ServerName, PachubeKey, strlen(buf), buf );
      
		// Send the blob
		TCPWrite( MySocket, tmpString, (int)strlen(tmpString) );

		#if defined(STACK_USE_UART)
		UARTWrite(1,"Data sent, Waiting response.\r\n");
		#endif
		vTaskDelay( xDelay(500) );

		// Make sure there are enough chars to read
		i = 10;
		while (i>0 && (TCPRxLen(MySocket) < 16))
		{
			vTaskDelay( xDelay(500) );
			i--;
		}
		if(i!=0) 
		{
			// Get the response from the server
			TCPRead( MySocket, tmpString, 16 );

			UARTWrite(1,tmpString);
			UARTWrite(1,"\r\n");

			if (strstr( tmpString, GoodHTTPResponse) == NULL)
			{
				// We are in an error condition, so light up the Flyport led
				err = TRUE;
				IOPut(o4,ON);
				#if defined(STACK_USE_UART)
				UARTWrite(1,"Request failed.\r\n");
				#endif
			} else 
			{
				#if defined(STACK_USE_UART)
				UARTWrite(1,"Request succeeded.\r\n");
				#endif
			}
		}
		#if defined(STACK_USE_UART)
		else 
		{
			UARTWrite(1,"Server did not reply.\r\n");
		}
		#endif

		#if defined(STACK_USE_UART)
		UARTWrite(1,"Disconnecting...");
		#endif
		TCPClientClose(MySocket);
		MySocket = INVALID_SOCKET;
		#if defined(STACK_USE_UART)
		UARTWrite(1,"Done.\r\n\r\n");
		#endif
	  
	    IOPut(o4,OFF);		// turn off LED= end of pachube connection
	}
}
